import unittest

from bet_optimizer import get_positive_odds, kelly_criterion_bet


class TestKellyCriterionBet(unittest.TestCase):
    def test_positive_cases(self):
        # Positive test cases
        self.assertAlmostEqual(kelly_criterion_bet(0.5, 2.0, 100), 25.00)
        self.assertAlmostEqual(kelly_criterion_bet(0.7, 1.5, 100), 50.00)
        self.assertAlmostEqual(kelly_criterion_bet(0.8, 1.2, 1000), 633.3333333333333)

    def test_negative_cases(self):
        # Negative test cases
        self.assertEqual(kelly_criterion_bet(-0.5, 2.0, 100), 0)
        self.assertEqual(kelly_criterion_bet(0.5, -2.0, 100), 0)
        self.assertEqual(kelly_criterion_bet(0.5, 2.0, -100), 0)
        self.assertEqual(kelly_criterion_bet(1.5, 2.0, 100), 0)


class TestGetPositiveOdds(unittest.TestCase):
    def test_positive_cases(self):
        # Positive test cases
        self.assertAlmostEqual(get_positive_odds(0.5), 1.0)
        self.assertAlmostEqual(get_positive_odds(0.25), 3.0)
        self.assertAlmostEqual(get_positive_odds(0.75), 0.3333333333333333)

    def test_negative_cases(self):
        # Negative test cases
        self.assertEqual(get_positive_odds(0), float("inf"))
        self.assertEqual(get_positive_odds(1), 0)
        self.assertEqual(get_positive_odds(-0.5), None)
        self.assertEqual(get_positive_odds(1.5), None)


if __name__ == "__main__":
    unittest.main()



def kelly_criterion_bet(probability: float, odds: float, bankroll: float = 100) -> float:
    """
    Calculate the Kelly Criterion bet size.

    Args:
        probability: The probability of winning the bet (0 to 1).
        odds: The odds received on the bet (in decimal form, e.g., 1.0 for even money).
        bankroll: The current size of the bankroll.

    Returns:
        The optimal fraction of the bankroll to bet.
    """
    if not 0 <= probability <= 1:
        print("Probability must be between 0 and 1")
        return 0
    if odds <= 0:
        print("Odds must be greater than 0")
        return 0
    if bankroll <= 0:
        print("Bankroll must be greater than 0")
        return 0

    complement = 1 - probability
    bet_size = (probability * odds - complement) / odds

    return bet_size * bankroll


def get_positive_odds(probability: float):
    """
    Calculate the minimum positive odds required for a bet to have a positive expected value.

    Args:
        probability (float): The probability of winning the bet (0 to 1).

    Returns:
        float: The minimum positive odds required for a bet to have a positive expected value.
               Returns infinity if the probability is 0, and None if the probability is outside the valid range.
    # ev = p * odds - (1-p) * 1
    # p * odds -1 + p > 0  # make ev positive
    # p(odds + 1) > 1
    # odds > (1/p) - 1
    """
    if probability == 0:
        return float("inf")
    if not 0 <= probability <= 1:
        return None

    return (1 - probability) / probability



from setuptools import setup, find_packages

setup(
    name='bet_optimizer',  # Name of package
    version='0.0.2',  # Version number
    packages=find_packages(),  # List of all Python modules to be installed
    description='Python script for optimizing bets',  # Short description of library
    long_description=open('README.md').read(),  # Long description read from the the readme file
    long_description_content_type='text/markdown',  # Type of the long description
    url='https://github.com/subodh101/bet_optimizer',  # Link to your github repository or project on another platform
    author='Subodh Pushkar',  # name
    author_email='subodh.pushkar@gmail.com',  # email
    license='Apache License',  # license
    install_requires=[],  # List of dependencies
    python_requires='>=3.6',  # Minimum version requirement of Python
    py_modules=['bet_optimizer'],  # Name of the python file you want to install
    classifiers=[
        # Trove classifiers
        # Full list available at https://pypi.org/classifiers/
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
)
